var bookmarkForTag = new Array();

//
// Contains the main logic
//
function DeliciousService(aGlobals)
{
  this.globals = aGlobals;
}

var urlDeliciousAPI = "https://api.del.icio.us/v1/";

DeliciousService.prototype = 
{

	urlPosts: this.urlDeliciousAPI + "posts/dates?",
	urlTags: this.urlDeliciousAPI + "tags/get?",
	urlBundles: this.urlDeliciousAPI + "tags/bundles/all",
	urlApiPost: this.urlDeliciousAPI + "posts/add?",
	urlApiGetAll: this.urlDeliciousAPI + "posts/all?",
	urlGetBookmark: this.urlDeliciousAPI + "posts/get?",
	urlRenameBookmark: this.urlDeliciousAPI + "tags/rename?old=",
	urlDeleteBookmark: this.urlDeliciousAPI + "posts/delete?url=",

	isFirstTimeKey: 'delicious.isFirstTime',
	postUrlKey: 'delicious.postUrlKey',
	postPopupUrlKey: 'delicious.postPopupUrlKey',
	
	defaultPostUrl: 'http://del.icio.us/$username?url=$url&title=$title&extended=$extended',	
	defaultPopupPostUrl: 'http://del.icio.us/$username?noui=yes&jump=close&url=$url&title=$title&extended=$extended',
  
  
    //
    // Hook deliciousService to current user manager.
    //
 	hookUserManager: function(userManager)
 	{
 	    this.userManager = userManager;
 	},
 	
	//
	// Get the URL of the del.icio.us post page
	//
	getPostUrl: function(username, url, title, extended)
	{
	    var postUrl = nsPreferences.copyUnicharPref(this.postUrlKey, "");   
	     
		// Use default if not specified
	    if(postUrl.length == 0)
	    {
    		nsPreferences.setUnicharPref(this.postUrlKey, this.defaultPostUrl);
    		postUrl = this.defaultPostUrl;
	    }
	    
	    postUrl = this.replaceVariables(postUrl, username, url, title, extended);
	    
	    return postUrl;
	},
	
	//
	// Get the URL of the del.icio.us popup post page
	//
	getPopupPostUrl: function(username, url, title, extended)
	{
	    var postUrl = nsPreferences.copyUnicharPref(this.postPopupUrlKey, "");    
		// Use default if not specified
	    if(postUrl.length == 0)
	    {
    		nsPreferences.setUnicharPref(this.postPopupUrlKey, this.defaultPopupPostUrl);
    		postUrl = this.defaultPopupPostUrl;
	    }

	    postUrl = this.replaceVariables(postUrl, username, url, title, extended);
	    
	    return postUrl;
	},

	//
	// Posts a URL using the del.icio.us REST API
	//
	postURL: function(url, description, extended, tags, date, isPrivate, asyncGetURLRequester)
	{
    	var username = this.getUsername();
    	if (username.length == 0) {
    	    return;
    	}
	    var password = this.getPassword();
    	if (password.length == 0) {
    	    return;
    	}
		var queryString = "&url=" + encodeURIComponent(url)
            + "&description=" + encodeURIComponent(description)
            + "&extended=" + encodeURIComponent(extended)
            + "&tags=" + encodeURIComponent(tags)
            + "&dt=" + date
            + "&replace=yes";

		if (isPrivate == true) {
			queryString = queryString + "&shared=no";
		}

    var urlToSend =this.urlApiPost + queryString;

    asyncGetURL(urlToSend, username, password, false, asyncGetURLRequester);
	},
	
	//
	// Reads all bookmarks for a specific tag using the delicious API
	//
	getAllBookmarks: function(tags, asyncGetURLRequester) {
		var username = this.getUsername();
		if (username.length == 0) {
				return null;
		}
		var password = this.getPassword();
		if (password.length == 0) {
				return null;
		}
    	    	
    var currentTag = tags.split(" ")[0];
    
		var url = this.urlApiGetAll + "&tag=" + encodeURIComponent(currentTag);
		
		// currectly working synchronously to easyly filter response
		//asyncGetURL(url, username, password, false, asyncGetURLRequester);
		
		var resp = null;
		if (bookmarkForTag[currentTag] != null) {
			resp = bookmarkForTag[currentTag];
		} else {		
			resp = getURL(url, username, password, false);		
			bookmarkForTag[currentTag] = resp;
		}
		
		var filteredResp = this.filterResponse(resp, tags);
		
	 	return filteredResp;
	},
	
	/**
	 * Filters the response xml so that it contains only posts which are associated all tags.
	 */
	filterResponse: function(response, tags) {			
		var returnResponse = "";
	
		searchString = response;
	
		returnResponse = searchString.substring(0, searchString.indexOf("<post "));
		var splitTags = tags.split(" ");
		while(searchString.indexOf("<post ") >= 0) {
			var postStart = searchString.indexOf("<post ");
			var postEnd = searchString.indexOf("/>") + 2;
			
			var postString = searchString.substring(postStart, postEnd);

			var postContainsAllTags = true;
			for (var i = 0; i < splitTags.length; i++) {	
				if (postString.indexOf(splitTags[i]) < 0) {
					postContainsAllTags = false;
					break;
				}
			}
			
			if (postContainsAllTags) {
				returnResponse += postString;
			}
	
			searchString = searchString.substring(postEnd + 1);
		}
	
	
		returnResponse += response.substring(response.lastIndexOf("/>") + 2, response.length - 1);		
	
		return returnResponse;
	},
	
	/*
	 * Read the bookmark for a specific url.
	 * Returns either bookmark-text or null
	 * TODO this guy should work with date and tag as well.
	 */
	getBookmarkByURL: function(url)
	{
    	var username = this.getUsername();
    	if (username.length == 0) {
    	    return null;
    	}
	    var password = this.getPassword();
    	if (password.length == 0) {
    	    return null;
    	}    	
    	
    	var urlToSend = this.urlGetBookmark;		
    	
    	if (url != null) {
    	 urlToSend += "&url=" + encodeURIComponent(url);
    	 return getURL(urlToSend, username, password, false);
    	}
    	
    	return null;
    
	},
	//
	// Get the tags as an XML document
	//
	getTags: function(aAsyncGetURLRequester)
	{
	    if (this.globals.userManager.getUsers().length == 0) return new DOMParser().parseFromString("<tags/>", "text/xml");
	    var username = this.getUsername();
	    if(username.length == 0) return new DOMParser().parseFromString("<tags/>", "text/xml");
	 	var password = this.getPassword();
	 	return this.getTagsForUser(aAsyncGetURLRequester, username, password);
	},
	
	//
	// Get the tags as an XML document
	//
	getTagsForUser: function(aAsyncGetURLRequester, aUsername, aPassword)	{
	  // reset cached data structure!
		bookmarkForTag = new Array();
	
	 	if (aAsyncGetURLRequester == null) {
        return getURL(this.urlTags, aUsername, aPassword, true);
    } else {
        asyncGetURL(this.urlTags, aUsername, aPassword, true, aAsyncGetURLRequester);            
    }
    return null;
	},
	//
	// Get the bundles as an XML document
	//
	getBundles: function(aAsyncGetURLRequester) {
		if (this.globals.userManager.getUsers().length == 0) {
			return null;
		}
	  
	  var username = this.getUsername();
	  if(username.length == 0) {
	  	return null;
	  }	
	  
	 	var password = this.getPassword();
	 	return this.getBundlesForUser(aAsyncGetURLRequester, username, password);
	},
	
	//
	// Get the tags as an XML document
	//
	getBundlesForUser: function(aAsyncGetURLRequester, aUsername, aPassword)
	{
	 	if (aAsyncGetURLRequester == null) {
        return getURL(this.urlBundles, aUsername, aPassword, false);
    } else {
        asyncGetURL(this.urlBundles, aUsername, aPassword, false, aAsyncGetURLRequester);            
    }
    return null;
	},	

	//
	// Returns the del.icio.us username or empty string if not specified.
	// But first it prompts the user for the username if it has not been specified
	//
	getUsername: function()
	{
	    var username = this.userManager.getDefaultUser().getUsername();
	    if (username.length == 0)
	    {
	        alert("You have to specify a del.icio.us username");
	        this.showPreferences();
	        username = this.userManager.getDefaultUser().getUsername();
	    }
	    return username;
	},
	
	//
	//
	//
	getPassword: function()
	{
	    var password = this.userManager.getDefaultUser().getPassword();
	    if(password.length == 0)
	    {
	        alert("You have to specify a del.icio.us password");
	        this.showPreferences();
	        password = this.userManager.getDefaultUser().getPassword();
	    }
	    return password;
	},
	
	//
	// 
	// 
	getUsePopup: function()
	{
	    var usePopup = nsPreferences.getBoolPref(this.usePopupKey, false);
	    return usePopup;
	},
	
	//
	//
	//
	setUsePopup: function(enabled)
	{
    	nsPreferences.setBoolPref(this.usePopupKey, enabled);
	},
	
	//
	// Posts a URL through with a popup pointing to del.icio.us
	//
	post: function(url, title, extended)
	{
	    // Get the username, default is empty
	    var username = this.getUsername();

	    if(username.length>0)
        {
            var usePopup = this.getUsePopup();
            var postUrl;
            
            // Check if we want should use the popup
            if(usePopup)
            {
                postUrl = this.getPopupPostUrl(username, url, title, extended);

                open(postUrl, "delicious", "toolbar=no,width=700,height=250");
            }
            else
            {
                postUrl = this.getPostUrl(username, url, title, extended);

                window._content.document.location = postUrl;
            }
        }
	},

	//
	// Opens the delicious extension's preferences box
	//
	showPreferences: function()
	{
		var preferencesDialog = window.openDialog('chrome://delicious/content/preferenceDialog.xul', 'del.icio.us preferences', "modal,centerscreen,chrome,resizable", {"globals":this.globals});
	},
	
	
	//
	// Open the user's del.icio.us page
	//
	showMyDelicious: function()
	{
	    var username = this.getUsername();
	    
	    if(username.length == 0) return;

	    var deliciousUrl = "http://del.icio.us/" + username;
	    window._content.document.location = deliciousUrl;
	},
	
	//
	// Open page showing the user's posts
	//
/*
  Functionality was removed from del.icio.us. See: http://mozdev.org/bugs/show_bug.cgi?id=8822

	showPosts: function()
	{
		// Checks if username & password is set
	    var username = this.getUsername();
	    var password = this.getPassword();

	    if(username.length == 0 || password.length == 0) return;

	    var postsDialog = window.openDialog('chrome://delicious/content/postsDialog.xul', 'del.icio.us', "modal,centerscreen,chrome,resizable=no", window);
	},
	
	//
	// Get the posts as an XML document
	//
	getPosts: function(aAsyncGetURLRequester)
	{
    	var username = this.getUsername();
	    var password = this.getPassword();
	    return getURL(this.urlPosts, username, password, true);
	},	
*/

	//
	// Show list of tags
	//
	showTags: function()
	{
		// Checks if username & password is set
	    var username = this.getUsername();
	    var password = this.getPassword();

	    if(username.length == 0 || password.length == 0) return;

    	var tagsDialog = window.openDialog('chrome://delicious/content/tagsDialog.xul', 'del.icio.us tags', "centerscreen,chrome,resizable", window, {userManager:this.userManager, deliciousService:this});
	},
	
	//
	// Opens the user's inbox
	//
	showInbox: function()
	{
	    var username = this.getUsername();

	    if(username.length == 0)
	    {
		    return;
	    }
	    var deliciousUrl = "http://del.icio.us/inbox/" + username;
	    window._content.document.location = deliciousUrl;
	},
	
	//
	// Displays the selected posts for a date
	//
	showSelectedDate:function (date, target)
	{
	    var username = this.getUsername();
	   
	    var deliciousUrl;
	
	    deliciousUrl = "http://del.icio.us/" + username + "/" + date;
	    target.location.href = deliciousUrl;
	},
	
	//
	// Displays the selected tag
	//
	showSelectedTag:function(tag, allUsers, target)
	{
    	var deliciousUrl="http://del.icio.us/";
    
	    if(allUsers) deliciousUrl += "tag";
	    else deliciousUrl += this.getUsername();
        
        deliciousUrl+="/\"" + tag + '\"';
    
        target.location.href = deliciousUrl;
	},
	
	//
	// Replaces variables in the post URL
	//
	replaceVariables:function(postUrl, username, url, title, extended)
	{
		if(username.length == 0)
			username = "";
		if(url.length == 0)
			url = "";
		if(title.length == 0)
			title = "";
		if(extended.length == 0)
			extended = "";
	
	    // Replace the variables
	    postUrl = postUrl.replace("$username", username);
	    postUrl = postUrl.replace("$url", url);
	    postUrl = postUrl.replace("$title", title);
	    postUrl = postUrl.replace("$extended", extended);
	    return postUrl;
	},
    
    //
    // Finds the current page in del.icio.us
    //
    findURL:function(url)
    {
        var findUrl = "http://del.icio.us/url/" + hex_md5(url);
        window._content.document.location = findUrl;
    },
    
    //
    // Rename a tag
    //
    renameTag:function(oldName, newName, aRequester)
    {
		var url = this.urlRenameBookmark;
		url += oldName;
		url += "&new=" + newName;
    	var username = this.getUsername();
	    var password = this.getPassword();
	    asyncGetURL(url, username, password, true, aRequester);
    },
    
    //
    // Remove a bookmark
    //
    deleteBookmark:function(bookmark, aRequester)
    {
			var url = this.urlDeleteBookmark;
			url += bookmark;
		
    	var username = this.getUsername();
	    var password = this.getPassword();
	    asyncGetURL(url, username, password, true, aRequester);
    },    
    
    /**
     * Login to the del.icio.us service.
     * Return true for success, false for failed login.
     */
    login: function()
    {
	    var username = this.getUsername();
	    var password = this.getPassword();
	    if(username.length == 0 || password.length == 0) {
	        return false;
	    }
        // Need cookies turned on for logging in.
        // The first test is for Firefox, the second for Thunderbird.
        if (!globals.cookies.managementEnabled || globals.cookies.turnCookiesOn() == false) {
            return false;
        }

        // Login to del.icio.us.        
        var loginURL = "https://secure.del.icio.us/login";
        var data = "user_name="+encodeURIComponent(username)+"&password="+encodeURIComponent(password)+"&login=log+in"
        asyncPostURL(loginURL, "", "", false, this.loginRequester, data);
        return true;
    },

    loginRequester: {
        // Process the asynchronous getting of the experimental fields info.
        asyncGetURLResponse: function(resp) {
            globals.cookies.restoreCookieState();
            if (resp == null || /action\s*=\s*.\/login/.test(resp) == true) {
                alert("Del.icio.us extension couldn't login to del.icio.us. Please check username and password.");
            }
        },

        // Process error.
        asyncGetURLError: function(error) {
            globals.cookies.restoreCookieState();
            return;
        }
    }
}
